USE AdventureWorks2014;
SET ANSI_NULLS ON;
SET QUOTED_IDENTIFIER ON;
SET NOCOUNT ON;
GO
-- SCHEMA TO SUPPORT JOB DURATION METRICS COLLECTION

-- Create a table to store job metadata.
CREATE TABLE dbo.dim_sql_agent_job
(	dim_sql_agent_job_Id INT NOT NULL IDENTITY(1,1) CONSTRAINT PK_dim_sql_agent_job PRIMARY KEY CLUSTERED,
	Sql_Agent_Job_Id UNIQUEIDENTIFIER NOT NULL,
	Sql_Agent_Job_Name NVARCHAR(128) NOT NULL,
	Job_Create_Datetime DATETIME NOT NULL, 
	Job_Last_Modified_Datetime DATETIME NOT NULL,
	Is_Enabled BIT NOT NULL,
	Is_Deleted BIT NOT NULL
);
GO
CREATE NONCLUSTERED INDEX IX_dim_sql_agent_job_Sql_Agent_Job_Id ON dbo.dim_sql_agent_job (Sql_Agent_Job_Id);
GO
-- Create a table to store job schedule metadata.
CREATE TABLE dbo.dim_sql_agent_schedule
(	Schedule_Id INT NOT NULL CONSTRAINT PK_fact_sql_agent_job_schedule PRIMARY KEY CLUSTERED,
	Schedule_Name NVARCHAR(128) NOT NULL,
	Is_Enabled BIT NOT NULL,
	Is_Deleted BIT NOT NULL,
	Schedule_Start_Date DATE NOT NULL,
	Schedule_End_Date DATE NOT NULL,
	Schedule_Occurrence VARCHAR(25) NOT NULL,
	Schedule_Occurrence_Detail VARCHAR(256) NULL, 
	Schedule_Frequency VARCHAR(256) NULL,
	Schedule_Created_Datetime DATETIME NOT NULL,
	Schedule_Last_Modified_Datetime DATETIME NOT NULL
);
GO
CREATE NONCLUSTERED INDEX IX_dim_sql_agent_schedule_Schedule_Last_Modified_Datetime ON dbo.dim_sql_agent_schedule (Schedule_Last_Modified_Datetime);
GO
-- Create a table to store associations between jobs and schedules.
CREATE TABLE dbo.fact_sql_agent_schedule_assignment
(	fact_sql_agent_schedule_assignment_id INT NOT NULL IDENTITY(1,1) CONSTRAINT PK_fact_sql_agent_schedule_assignment PRIMARY KEY CLUSTERED,
	Sql_Agent_Job_Id UNIQUEIDENTIFIER NOT NULL,
	Schedule_Id INT NOT NULL,
	Next_Run_Datetime DATETIME NULL
);
GO
CREATE NONCLUSTERED INDEX IX_fact_sql_agent_schedule_assignment_Sql_Agent_Job_Id ON dbo.fact_sql_agent_schedule_assignment (Sql_Agent_Job_Id);
GO
-- Create the table to store job run time data.
CREATE TABLE dbo.fact_job_run_time
(	Sql_Agent_Job_Run_Instance_Id INT NOT NULL CONSTRAINT PK_fact_job_run_time PRIMARY KEY CLUSTERED,
	Sql_Agent_Job_Id UNIQUEIDENTIFIER NOT NULL,
	Job_Start_Datetime DATETIME NOT NULL,
	Job_End_Datetime AS DATEADD(SECOND, job_duration_seconds, Job_Start_Datetime) PERSISTED,
	Job_Duration_Seconds INT NOT NULL,
	Job_Status VARCHAR(8) NOT NULL
);

CREATE NONCLUSTERED INDEX IX_fact_job_run_time_job_start_time ON dbo.fact_job_run_time (Job_Start_Datetime);
CREATE NONCLUSTERED INDEX IX_fact_job_run_time_Sql_Agent_Job_Id ON dbo.fact_job_run_time (Sql_Agent_Job_Id);
GO
-- Create the table to store job step run time data.
CREATE TABLE dbo.fact_step_job_run_time
(	Sql_Agent_Job_Run_Instance_Id INT NOT NULL CONSTRAINT PK_fact_step_job_run_time PRIMARY KEY CLUSTERED,
	Sql_Agent_Job_Id UNIQUEIDENTIFIER NOT NULL,
	Sql_Agent_Job_Step_Id INT NOT NULL,
	Job_Step_Start_Datetime DATETIME NOT NULL,
	Job_Step_End_Datetime AS DATEADD(SECOND, Job_Step_Duration_seconds, Job_Step_Start_Datetime) PERSISTED,
	Job_Step_Duration_seconds INT NOT NULL,
	Job_Step_Status VARCHAR(8) NOT NULL
);

CREATE NONCLUSTERED INDEX IX_fact_step_job_run_time_Sql_Agent_Job_Id_Sql_Agent_Job_Step_Id ON dbo.fact_step_job_run_time (Sql_Agent_Job_Id, Sql_Agent_Job_Step_Id);
CREATE NONCLUSTERED INDEX IX_fact_step_job_run_time_job_step_start_time ON dbo.fact_step_job_run_time (Job_Step_Start_Datetime);
GO

-- Create a stored procedure that will populate and maintain the tables created above.
IF EXISTS (SELECT * FROM sys.procedures WHERE procedures.name = 'usp_get_job_execution_metrics')
BEGIN
	DROP PROCEDURE dbo.usp_get_job_execution_metrics;
END
GO

CREATE PROCEDURE dbo.usp_get_job_execution_metrics
AS
BEGIN
	SET NOCOUNT ON;
	-- Determine last date/time that a schedule was modified within our data.
	DECLARE @Schedule_Data_Last_Modify_Datetime DATETIME;
	SELECT
		@Schedule_Data_Last_Modify_Datetime = MAX(dim_sql_agent_schedule.Schedule_Last_Modified_Datetime)
	FROM dbo.dim_sql_agent_schedule;
	IF @Schedule_Data_Last_Modify_Datetime IS NULL
	BEGIN
		SELECT @Schedule_Data_Last_Modify_Datetime = '1/1/1900';
	END
	-- Collect all schedule additions/changes since the last update.
	MERGE INTO dbo.dim_sql_agent_schedule AS SCHEDULE_TARGET
		USING	(
			SELECT
				sysschedules.schedule_id,
				sysschedules.name AS Schedule_Name,
				sysschedules.enabled AS Is_Enabled,
				0 AS Is_Deleted,
				CAST(SUBSTRING(CAST(sysschedules.active_start_date AS VARCHAR(MAX)), 5, 2) + '/' + SUBSTRING(CAST(sysschedules.active_start_date AS VARCHAR(MAX)), 7, 2) + '/' + SUBSTRING(CAST(sysschedules.active_start_date AS VARCHAR(MAX)), 1, 4) AS DATE) AS Schedule_Start_Date,
				CAST(SUBSTRING(CAST(sysschedules.active_end_date AS VARCHAR(MAX)), 5, 2) + '/' + SUBSTRING(CAST(sysschedules.active_end_date AS VARCHAR(MAX)), 7, 2) + '/' + SUBSTRING(CAST(sysschedules.active_end_date AS VARCHAR(MAX)), 1, 4) AS DATE) AS Schedule_End_Date,
				CASE sysschedules.freq_type
							WHEN 1 THEN 'Once'
							WHEN 4 THEN 'Daily'
							WHEN 8 THEN 'Weekly'
							WHEN 16 THEN 'Monthly'
							WHEN 32 THEN 'Monthly (relative)'
							WHEN 64 THEN 'At Agent Startup'
							WHEN 128 THEN 'When CPU(s) idle' 
				END AS Schedule_Occurrence,
				CASE sysschedules.freq_type
					WHEN 1 THEN 'Once'
					WHEN 4 THEN 'Every ' + CONVERT(VARCHAR, sysschedules.freq_interval) + ' day(s)'
					WHEN 8 THEN 'Every ' + CONVERT(VARCHAR, sysschedules.freq_recurrence_factor) + ' weeks(s) on ' + 
						LEFT(	CASE WHEN sysschedules.freq_interval &  1 =  1 THEN 'Sunday, '    ELSE '' END + 
								CASE WHEN sysschedules.freq_interval &  2 =  2 THEN 'Monday, '    ELSE '' END + 
								CASE WHEN sysschedules.freq_interval &  4 =  4 THEN 'Tuesday, '   ELSE '' END + 
								CASE WHEN sysschedules.freq_interval &  8 =  8 THEN 'Wednesday, ' ELSE '' END + 
								CASE WHEN sysschedules.freq_interval & 16 = 16 THEN 'Thursday, '  ELSE '' END + 
								CASE WHEN sysschedules.freq_interval & 32 = 32 THEN 'Friday, '    ELSE '' END + 
								CASE WHEN sysschedules.freq_interval & 64 = 64 THEN 'Saturday, '  ELSE '' END , 
							LEN(	CASE WHEN sysschedules.freq_interval &  1 =  1 THEN 'Sunday, '    ELSE '' END + 
									CASE WHEN sysschedules.freq_interval &  2 =  2 THEN 'Monday, '    ELSE '' END + 
									CASE WHEN sysschedules.freq_interval &  4 =  4 THEN 'Tuesday, '   ELSE '' END + 
									CASE WHEN sysschedules.freq_interval &  8 =  8 THEN 'Wednesday, ' ELSE '' END + 
									CASE WHEN sysschedules.freq_interval & 16 = 16 THEN 'Thursday, '  ELSE '' END + 
									CASE WHEN sysschedules.freq_interval & 32 = 32 THEN 'Friday, '    ELSE '' END + 
									CASE WHEN sysschedules.freq_interval & 64 = 64 THEN 'Saturday, '  ELSE '' END) - 1)
					WHEN  16 THEN 'Day ' + CONVERT(VARCHAR, sysschedules.freq_interval) + ' of every ' + CONVERT(VARCHAR, sysschedules.freq_recurrence_factor) + ' month(s)'
					WHEN  32 THEN 'The ' +	CASE sysschedules.freq_relative_interval
												WHEN 1 THEN 'First'
												WHEN 2 THEN 'Second'
												WHEN 4 THEN 'Third'
												WHEN 8 THEN 'Fourth'
												WHEN 16 THEN 'Last'
											END +
											CASE sysschedules.freq_interval
												WHEN 1 THEN ' Sunday'
												WHEN 2 THEN ' Monday'
												WHEN 3 THEN ' Tuesday'
												WHEN 4 THEN ' Wednesday'
												WHEN 5 THEN ' Thursday'
												WHEN 6 THEN ' Friday'
												WHEN 7 THEN ' Saturday'
												WHEN 8 THEN ' Day'
												WHEN 9 THEN ' Weekday'
												WHEN 10 THEN ' Weekend Day' 
											END + ' of every ' + CONVERT(VARCHAR, sysschedules.freq_recurrence_factor) + ' month(s)' 
				END AS Schedule_Occurrence_Detail,
				CASE sysschedules.freq_subday_type
					WHEN 1 THEN 'Occurs once at ' + 
								STUFF(STUFF(RIGHT('000000' + CONVERT(VARCHAR(8), sysschedules.active_start_time), 6), 5, 0, ':'), 3, 0, ':')
					WHEN 2 THEN 'Occurs every ' + 
								CONVERT(VARCHAR, sysschedules.freq_subday_interval) + ' Seconds(s) between ' + 
								STUFF(STUFF(RIGHT('000000' + CONVERT(VARCHAR(8), sysschedules.active_start_time), 6), 5, 0, ':'), 3, 0, ':') + ' and ' + 
								STUFF(STUFF(RIGHT('000000' + CONVERT(VARCHAR(8), sysschedules.active_end_time), 6), 5, 0, ':'), 3, 0, ':')
					WHEN 4 THEN 'Occurs every ' + 
								CONVERT(VARCHAR, sysschedules.freq_subday_interval) + ' Minute(s) between ' + 
								STUFF(STUFF(RIGHT('000000' + CONVERT(VARCHAR(8), sysschedules.active_start_time), 6), 5, 0, ':'), 3, 0, ':') + ' and ' + 
								STUFF(STUFF(RIGHT('000000' + CONVERT(VARCHAR(8), sysschedules.active_end_time), 6), 5, 0, ':'), 3, 0, ':')
					WHEN 8 THEN 'Occurs every ' + 
								CONVERT(VARCHAR, sysschedules.freq_subday_interval) + ' Hour(s) between ' + 
								STUFF(STUFF(RIGHT('000000' + CONVERT(VARCHAR(8), sysschedules.active_start_time), 6), 5, 0, ':'), 3, 0, ':') + ' and ' + 
								STUFF(STUFF(RIGHT('000000' + CONVERT(VARCHAR(8), sysschedules.active_end_time), 6), 5, 0, ':'), 3, 0, ':')
				END AS Schedule_Frequency,
				sysschedules.date_created AS Schedule_Created_Datetime,
				sysschedules.date_modified AS Schedule_Last_Modified_Datetime
			FROM msdb.dbo.sysschedules
			WHERE sysschedules.date_modified > @Schedule_Data_Last_Modify_Datetime) AS SCHEDULE_SOURCE
		ON (SCHEDULE_SOURCE.schedule_id = SCHEDULE_TARGET.schedule_id)
		WHEN NOT MATCHED BY TARGET
			THEN INSERT
				(Schedule_Id, Schedule_Name, Is_Enabled, Is_Deleted, Schedule_Start_Date, Schedule_End_Date, Schedule_Occurrence, Schedule_Occurrence_Detail, Schedule_Frequency, Schedule_Created_Datetime, Schedule_Last_Modified_Datetime)
			VALUES	(
				SCHEDULE_SOURCE.Schedule_Id,
				SCHEDULE_SOURCE.Schedule_Name,
				SCHEDULE_SOURCE.Is_Enabled,
				SCHEDULE_SOURCE.Is_Deleted,
				SCHEDULE_SOURCE.Schedule_Start_Date,
				SCHEDULE_SOURCE.Schedule_End_Date,
				SCHEDULE_SOURCE.Schedule_Occurrence,
				SCHEDULE_SOURCE.Schedule_Occurrence_Detail,
				SCHEDULE_SOURCE.Schedule_Frequency,
				SCHEDULE_SOURCE.Schedule_Created_Datetime,
				SCHEDULE_SOURCE.Schedule_Last_Modified_Datetime)
		WHEN MATCHED
			THEN UPDATE
				SET Schedule_Name = SCHEDULE_SOURCE.Schedule_Name,
					Is_Enabled = SCHEDULE_SOURCE.Is_Enabled,
					Is_Deleted = SCHEDULE_SOURCE.Is_Deleted,
					Schedule_Start_Date = SCHEDULE_SOURCE.Schedule_Start_Date,
					Schedule_End_Date = SCHEDULE_SOURCE.Schedule_End_Date,
					Schedule_Occurrence = SCHEDULE_SOURCE.Schedule_Occurrence,
					Schedule_Occurrence_Detail = SCHEDULE_SOURCE.Schedule_Occurrence_Detail,
					Schedule_Frequency = SCHEDULE_SOURCE.Schedule_Frequency,
					Schedule_Created_Datetime = SCHEDULE_SOURCE.Schedule_Created_Datetime,
					Schedule_Last_Modified_Datetime = SCHEDULE_SOURCE.Schedule_Last_Modified_Datetime;
	-- If a schedule was deleted, then mark it as no longer enabled.
	UPDATE dim_sql_agent_schedule
		SET Is_Enabled = 0,
			Is_Deleted = 1
	FROM dbo.dim_sql_agent_schedule
	LEFT JOIN msdb.dbo.sysschedules
	ON sysschedules.schedule_id = dim_sql_agent_schedule.Schedule_Id
	WHERE sysschedules.schedule_id IS NULL;
	-- Determine the last modify time for job data.
	DECLARE @Job_Data_Last_Modify_Datetime DATETIME;
	SELECT
		@Job_Data_Last_Modify_Datetime = MAX(dim_sql_agent_job.Job_Last_Modified_Datetime)
	FROM dbo.dim_sql_agent_job;
	IF @Job_Data_Last_Modify_Datetime IS NULL
	BEGIN
		SELECT @Job_Data_Last_Modify_Datetime = '1/1/1900';
	END
	-- Update our jobs data with any changes since the last update time.
	MERGE INTO dbo.dim_sql_agent_job AS JOB_TARGET
		USING	(
			SELECT
				sysjobs.job_id AS Sql_Agent_Job_Id,
				sysjobs.name AS Sql_Agent_Job_Name,
				sysjobs.date_created AS Job_Create_Datetime,
				sysjobs.date_modified AS Job_Last_Modified_Datetime,
				sysjobs.enabled AS Is_Enabled,
				0 AS Is_Deleted
			FROM msdb.dbo.sysjobs
			WHERE sysjobs.date_modified > @Job_Data_Last_Modify_Datetime) AS JOB_SOURCE
		ON (JOB_SOURCE.Sql_Agent_Job_Id = JOB_TARGET.Sql_Agent_Job_Id)
		WHEN NOT MATCHED BY TARGET
			THEN INSERT
				(Sql_Agent_Job_Id, Sql_Agent_Job_Name, Job_Create_Datetime, Job_Last_Modified_Datetime, Is_Enabled, Is_Deleted)
			VALUES	(
				JOB_SOURCE.Sql_Agent_Job_Id,
				JOB_SOURCE.Sql_Agent_Job_Name,
				JOB_SOURCE.Job_Create_Datetime,
				JOB_SOURCE.Job_Last_Modified_Datetime,
				JOB_SOURCE.Is_Enabled,
				JOB_SOURCE.Is_Deleted)
		WHEN MATCHED
			THEN UPDATE
				SET Sql_Agent_Job_Id = JOB_SOURCE.Sql_Agent_Job_Id,
					Sql_Agent_Job_Name = JOB_SOURCE.Sql_Agent_Job_Name,
					Job_Create_Datetime = JOB_SOURCE.Job_Create_Datetime,
					Job_Last_Modified_Datetime = JOB_SOURCE.Job_Last_Modified_Datetime,
					Is_Enabled = JOB_SOURCE.Is_Enabled,
					Is_Deleted = JOB_SOURCE.Is_Deleted;
	-- If a job was deleted, then mark it as no longer enabled.
	UPDATE dim_sql_agent_job
		SET Is_Enabled = 0,
			Is_Deleted = 1
	FROM dbo.dim_sql_agent_job
	LEFT JOIN msdb.dbo.sysjobs
	ON sysjobs.Job_Id = dim_sql_agent_job.Sql_Agent_Job_Id
	WHERE sysjobs.Job_Id IS NULL;

	-- Update Job/Schedule associations, and populate the next run time, as of the time of this entry
	MERGE INTO dbo.fact_sql_agent_schedule_assignment AS JOB_ASSIGNMENT_TARGET
		USING	(
			SELECT
				sysjobschedules.schedule_id,
				sysjobschedules.job_id
			FROM msdb.dbo.sysjobschedules) AS JOB_ASSIGNMENT_SOURCE
		ON (JOB_ASSIGNMENT_SOURCE.job_id = JOB_ASSIGNMENT_TARGET.Sql_Agent_Job_Id
		AND JOB_ASSIGNMENT_SOURCE.schedule_id = JOB_ASSIGNMENT_TARGET.schedule_id)
		WHEN NOT MATCHED BY TARGET
			THEN INSERT
				(Sql_Agent_Job_Id, Schedule_Id)
			VALUES	(
				JOB_ASSIGNMENT_SOURCE.job_id,
				JOB_ASSIGNMENT_SOURCE.schedule_id)
		WHEN NOT MATCHED BY SOURCE
			THEN DELETE;

	WITH CTE_LAST_RUN_TIME AS (
		SELECT
			sysjobactivity.job_id,
			MIN(sysjobactivity.next_scheduled_run_date) AS Next_Run_Datetime
		FROM msdb.dbo.sysjobactivity
		WHERE sysjobactivity.next_scheduled_run_date > CURRENT_TIMESTAMP
		GROUP BY sysjobactivity.job_id)
	UPDATE fact_sql_agent_schedule_assignment
		SET Next_Run_Datetime = CTE_LAST_RUN_TIME.Next_Run_Datetime
	FROM CTE_LAST_RUN_TIME
	INNER JOIN dbo.fact_sql_agent_schedule_assignment
	ON CTE_LAST_RUN_TIME.job_id = fact_sql_agent_schedule_assignment.Sql_Agent_Job_Id;

	-- Update job runtime metrics
	WITH CTE_NORMALIZE_DATETIME_DATA AS (
	SELECT
		sysjobhistory.instance_id AS Sql_Agent_Job_Run_Instance_Id,
		sysjobhistory.job_id AS Sql_Agent_Job_Id,
		CAST(sysjobhistory.run_date AS VARCHAR(MAX)) AS Run_Date_VARCHAR, 
		REPLICATE('0', 6 - LEN(CAST(sysjobhistory.run_time AS VARCHAR(MAX)))) + CAST(sysjobhistory.run_time AS VARCHAR(MAX)) AS Run_Time_VARCHAR,
		REPLICATE('0', 6 - LEN(CAST(sysjobhistory.run_duration AS VARCHAR(MAX)))) + CAST(sysjobhistory.run_duration AS VARCHAR(MAX)) AS Run_Duration_VARCHAR,
		sysjobhistory.run_status AS Run_Status
	FROM msdb.dbo.sysjobhistory
	WHERE sysjobhistory.instance_id NOT IN
		(SELECT fact_job_run_time.Sql_Agent_Job_Run_Instance_Id FROM dbo.fact_job_run_time)
	AND sysjobhistory.step_id = 0)
	INSERT INTO dbo.fact_job_run_time
		(Sql_Agent_Job_Run_Instance_Id, Sql_Agent_Job_Id, Job_Start_Datetime, Job_Duration_Seconds, Job_Status)
	SELECT
		CTE_NORMALIZE_DATETIME_DATA.Sql_Agent_Job_Run_Instance_Id,
		CTE_NORMALIZE_DATETIME_DATA.Sql_Agent_Job_Id,
		CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Date_VARCHAR, 5, 2) + '/' + SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Date_VARCHAR, 7, 2) + '/' + SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Date_VARCHAR, 1, 4) AS DATETIME) +
		CAST(STUFF(STUFF(CTE_NORMALIZE_DATETIME_DATA.Run_Time_VARCHAR, 5, 0, ':'), 3, 0, ':') AS DATETIME) AS Job_Start_Datetime,
		CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Duration_VARCHAR, 1, 2) AS INT) * 3600 +
			CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Duration_VARCHAR, 3, 2) AS INT) * 60 + 
			CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Duration_VARCHAR, 5, 2) AS INT) AS Job_Duration_Seconds,
		CASE CTE_NORMALIZE_DATETIME_DATA.Run_Status
			WHEN 0 THEN 'Failure'
			WHEN 1 THEN 'Success'
			WHEN 2 THEN 'Retry'
			WHEN 3 THEN 'Canceled'
			ELSE 'Unknown'
		END AS Job_Status
	FROM CTE_NORMALIZE_DATETIME_DATA;
	
	-- Update job step runtime metrics
	WITH CTE_NORMALIZE_DATETIME_DATA AS (
	SELECT
		sysjobhistory.instance_id AS Sql_Agent_Job_Run_Instance_Id,
		sysjobhistory.job_id AS Sql_Agent_Job_Id,
		sysjobhistory.step_id AS Sql_Agent_Job_Step_Id,
		CAST(sysjobhistory.run_date AS VARCHAR(MAX)) AS Run_Date_VARCHAR,
		REPLICATE('0', 6 - LEN(CAST(sysjobhistory.run_time AS VARCHAR(MAX)))) + CAST(sysjobhistory.run_time AS VARCHAR(MAX)) AS Run_Time_VARCHAR,
		REPLICATE('0', 6 - LEN(CAST(sysjobhistory.run_duration AS VARCHAR(MAX)))) + CAST(sysjobhistory.run_duration AS VARCHAR(MAX)) AS Run_Duration_VARCHAR,
		sysjobhistory.run_status AS Run_Status
	FROM msdb.dbo.sysjobhistory
	WHERE sysjobhistory.instance_id NOT IN
		(SELECT fact_step_job_run_time.Sql_Agent_Job_Run_Instance_Id FROM dbo.fact_step_job_run_time)
	AND sysjobhistory.step_id <> 0)
	INSERT INTO dbo.fact_step_job_run_time
		(Sql_Agent_Job_Run_Instance_Id, Sql_Agent_Job_Id, Sql_Agent_Job_Step_Id, Job_Step_Start_Datetime, Job_Step_Duration_seconds, Job_Step_Status)
	SELECT
		CTE_NORMALIZE_DATETIME_DATA.Sql_Agent_Job_Run_Instance_Id,
		CTE_NORMALIZE_DATETIME_DATA.Sql_Agent_Job_Id,
		CTE_NORMALIZE_DATETIME_DATA.Sql_Agent_Job_Step_Id,
		CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Date_VARCHAR, 5, 2) + '/' + SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Date_VARCHAR, 7, 2) + '/' + SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Date_VARCHAR, 1, 4) AS DATETIME) +
		CAST(STUFF(STUFF(CTE_NORMALIZE_DATETIME_DATA.Run_Time_VARCHAR, 5, 0, ':'), 3, 0, ':') AS DATETIME) AS Job_Step_Start_Datetime,
		CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Duration_VARCHAR, 1, 2) AS INT) * 3600 +
			CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Duration_VARCHAR, 3, 2) AS INT) * 60 + 
			CAST(SUBSTRING(CTE_NORMALIZE_DATETIME_DATA.Run_Duration_VARCHAR, 5, 2) AS INT) AS Job_Step_Duration_seconds,
		CASE CTE_NORMALIZE_DATETIME_DATA.Run_Status
			WHEN 0 THEN 'Failure'
			WHEN 1 THEN 'Success'
			WHEN 2 THEN 'Retry'
			WHEN 3 THEN 'Canceled'
			ELSE 'Unknown'
		END AS Job_Step_Status
	FROM CTE_NORMALIZE_DATETIME_DATA;
	-- Perform cleanup of the job table
	DELETE fact_job_run_time
	FROM dbo.fact_job_run_time
	WHERE fact_job_run_time.Job_Start_Datetime < DATEADD(YEAR, -1, CURRENT_TIMESTAMP);
	-- Perform cleanup of the job step table
	DELETE fact_step_job_run_time
	FROM dbo.fact_step_job_run_time
	WHERE fact_step_job_run_time.Job_Step_Start_Datetime < DATEADD(YEAR, -1, CURRENT_TIMESTAMP);
END
GO


/*	Queries to assist testing job performance data:

	-- Populate basic 5 tables of job duration metrics.
	EXEC dbo.usp_get_job_execution_metrics

	-- Select data from the tables populated above.
	SELECT * FROM dim_sql_agent_job
	SELECT * FROM dim_sql_agent_schedule
	SELECT * FROM fact_sql_agent_schedule_assignment
	SELECT * FROM fact_job_run_time;
	SELECT * FROM fact_step_job_run_time;

	-- Populate the aggregate job performance metrics table
	EXEC dbo.usp_populate_job_performance_metrics;

	-- Review data in the daily metrics table
	SELECT * FROM dbo.fact_daily_job_runtime_metrics

	-- Return data about our schedules.
	EXEC dbo.usp_report_job_schedule_details;
*/
